# -*- coding: utf-8 -*-
# Form implementation generated from reading ui file '/Users/goanpeca/Dropbox (Personal)/develop/tdk/spyder-remote/spyder-remote-client/spyder_remote_client/spyder/dialog.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!
from qtpy import QtCore
from qtpy import QtGui
from qtpy import QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(448, 354)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        Dialog.setMinimumSize(QtCore.QSize(0, 0))
        Dialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMinimumSize(QtCore.QSize(140, 25))
        self.label.setMaximumSize(QtCore.QSize(140, 25))
        self.label.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spyderHosts = QtWidgets.QComboBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(25)
        sizePolicy.setHeightForWidth(self.spyderHosts.sizePolicy().hasHeightForWidth())
        self.spyderHosts.setSizePolicy(sizePolicy)
        self.spyderHosts.setMinimumSize(QtCore.QSize(180, 25))
        self.spyderHosts.setObjectName("spyderHosts")
        self.spyderHosts.addItem("")
        self.spyderHosts.addItem("")
        self.spyderHosts.addItem("")
        self.spyderHosts.addItem("")
        self.spyderHosts.addItem("")
        self.horizontalLayout.addWidget(self.spyderHosts)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.credentials = QtWidgets.QGroupBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.credentials.sizePolicy().hasHeightForWidth())
        self.credentials.setSizePolicy(sizePolicy)
        self.credentials.setObjectName("credentials")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.credentials)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QtWidgets.QLabel(self.credentials)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setMinimumSize(QtCore.QSize(130, 25))
        self.label_4.setMaximumSize(QtCore.QSize(130, 25))
        self.label_4.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.user = QtWidgets.QComboBox(self.credentials)
        self.user.setObjectName("user")
        self.user.addItem("")
        self.user.addItem("")
        self.horizontalLayout_3.addWidget(self.user)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_5 = QtWidgets.QLabel(self.credentials)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setMinimumSize(QtCore.QSize(130, 25))
        self.label_5.setMaximumSize(QtCore.QSize(130, 25))
        self.label_5.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        self.password = QtWidgets.QLineEdit(self.credentials)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.password.sizePolicy().hasHeightForWidth())
        self.password.setSizePolicy(sizePolicy)
        self.password.setMinimumSize(QtCore.QSize(160, 25))
        self.password.setMaximumSize(QtCore.QSize(16777215, 25))
        self.password.setClearButtonEnabled(True)
        self.password.setObjectName("password")
        self.horizontalLayout_4.addWidget(self.password)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.keyring = QtWidgets.QCheckBox(self.credentials)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.keyring.sizePolicy().hasHeightForWidth())
        self.keyring.setSizePolicy(sizePolicy)
        self.keyring.setMinimumSize(QtCore.QSize(0, 25))
        self.keyring.setMaximumSize(QtCore.QSize(16777215, 25))
        self.keyring.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.keyring.setChecked(True)
        self.keyring.setObjectName("keyring")
        self.verticalLayout.addWidget(self.keyring)
        self.verticalLayout_3.addWidget(self.credentials)
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setMinimumSize(QtCore.QSize(130, 25))
        self.label_2.setMaximumSize(QtCore.QSize(130, 25))
        self.label_2.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.condaEnvironments = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(25)
        sizePolicy.setHeightForWidth(
            self.condaEnvironments.sizePolicy().hasHeightForWidth()
        )
        self.condaEnvironments.setSizePolicy(sizePolicy)
        self.condaEnvironments.setMinimumSize(QtCore.QSize(180, 25))
        self.condaEnvironments.setObjectName("condaEnvironments")
        self.condaEnvironments.addItem("")
        self.condaEnvironments.addItem("")
        self.condaEnvironments.addItem("")
        self.condaEnvironments.addItem("")
        self.condaEnvironments.addItem("")
        self.condaEnvironments.addItem("")
        self.horizontalLayout_2.addWidget(self.condaEnvironments)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setMinimumSize(QtCore.QSize(25, 25))
        self.label_3.setMaximumSize(QtCore.QSize(25, 25))
        self.label_3.setText("")
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setMinimumSize(QtCore.QSize(130, 25))
        self.label_6.setMaximumSize(QtCore.QSize(130, 25))
        self.label_6.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_6.addWidget(self.label_6)
        self.requirements = QtWidgets.QComboBox(self.groupBox)
        self.requirements.setObjectName("requirements")
        self.requirements.addItem("")
        self.horizontalLayout_6.addWidget(self.requirements)
        self.findRequirements = QtWidgets.QToolButton(self.groupBox)
        self.findRequirements.setMinimumSize(QtCore.QSize(25, 25))
        self.findRequirements.setMaximumSize(QtCore.QSize(25, 25))
        self.findRequirements.setObjectName("findRequirements")
        self.horizontalLayout_6.addWidget(self.findRequirements)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.feedback = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.feedback.sizePolicy().hasHeightForWidth())
        self.feedback.setSizePolicy(sizePolicy)
        self.feedback.setMinimumSize(QtCore.QSize(0, 25))
        self.feedback.setMaximumSize(QtCore.QSize(16777215, 25))
        self.feedback.setAlignment(QtCore.Qt.AlignCenter)
        self.feedback.setObjectName("feedback")
        self.verticalLayout_3.addWidget(self.feedback)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.cancelButton = QtWidgets.QPushButton(Dialog)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout_5.addWidget(self.cancelButton)
        spacerItem = QtWidgets.QSpacerItem(
            40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.horizontalLayout_5.addItem(spacerItem)
        self.connectButton = QtWidgets.QPushButton(Dialog)
        self.connectButton.setObjectName("connectButton")
        self.horizontalLayout_5.addWidget(self.connectButton)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "Remote Spyder Host :"))
        self.spyderHosts.setItemText(
            0, _translate("Dialog", "Gonzalo Peña-Castellanos' Raspberry Pi")
        )
        self.spyderHosts.setItemText(1, _translate("Dialog", "local"))
        self.spyderHosts.setItemText(2, _translate("Dialog", "Carlos' Mac Book pro"))
        self.spyderHosts.setItemText(3, _translate("Dialog", "Ralf's Server"))
        self.spyderHosts.setItemText(4, _translate("Dialog", "Tom's MiniSCT"))
        self.credentials.setTitle(_translate("Dialog", "Credentials :"))
        self.label_4.setText(_translate("Dialog", "User :"))
        self.user.setItemText(0, _translate("Dialog", "goanpeca"))
        self.user.setItemText(1, _translate("Dialog", "nerohmot"))
        self.label_5.setText(_translate("Dialog", "Password :"))
        self.password.setPlaceholderText(_translate("Dialog", "from keyring"))
        self.keyring.setText(_translate("Dialog", "Store/Update Password in keyring"))
        self.groupBox.setTitle(_translate("Dialog", "Conda :"))
        self.label_2.setText(_translate("Dialog", "Environment :"))
        self.condaEnvironments.setToolTip(
            _translate("Dialog", "Select the remote conda environment to work in")
        )
        self.condaEnvironments.setItemText(0, _translate("Dialog", "sandbox"))
        self.condaEnvironments.setItemText(1, _translate("Dialog", "base"))
        self.condaEnvironments.setItemText(2, _translate("Dialog", "anaconda3"))
        self.condaEnvironments.setItemText(3, _translate("Dialog", "_spyder_"))
        self.condaEnvironments.setItemText(4, _translate("Dialog", "MiniSCT"))
        self.condaEnvironments.setItemText(5, _translate("Dialog", "RPI"))
        self.label_6.setText(_translate("Dialog", "Requirements :"))
        self.requirements.setItemText(0, _translate("Dialog", "N/A"))
        self.findRequirements.setText(_translate("Dialog", "..."))
        self.feedback.setText(_translate("Dialog", "feedback line"))
        self.cancelButton.setText(_translate("Dialog", "Cancel"))
        self.connectButton.setText(_translate("Dialog", "Connect"))



#!/usr/bin/env python3
# -*- coding': utf-8 -*-
"""
Created on Sat Jun 20 09:21:26 2020

@author': nerohmot
"""
import os
import platform
import time

if platform.system() != "Windows":
    import fcntl  # is this linux only ?
import struct
import socket
import netifaces
import json
import urllib3
import OpenSSL

# from OpenSSL import crypto, SSL


def str2bool(string):
    """This function converts a string to a boolean."""
    return string.lower() in ("yes", "y", "true", "t", "1")


rIFF = {
    0x0001: "UP",  # Interface is up.
    0x0002: "BROADCAST",  # Broadcast address valid.
    0x0004: "DEBUG",  # Turn on debugging.
    0x0008: "LOOPBACK",  # Is a loopback net.
    0x0010: "POINTOPOINT",  # Interface is point-to-point link.
    0x0020: "NOTRAILERS",  # Avoid use of trailers.
    0x0040: "RUNNING",  # Resources allocated.
    0x0080: "NOARP",  # No address resolution protocol.
    0x0100: "PROMISC",  # Receive all packets.
    0x0200: "ALLMULTI",  # Receive all multicast packets.
    0x0400: "MASTER",  # Master of a load balancer.
    0x0800: "SLAVE",  # Slave of a load balancer.
    0x1000: "MULTICAST",  # Supports multicast.
    0x2000: "PORTSEL",  # Can set media type.
    0x4000: "AUTOMEDIA",  # Auto media select active.
    0x8000: "DYNAMIC",
}  # Dialup device with changing addresses.


def ifstatus(ifname):
    """This function returns the status of 'ifname' as a set of human readable flags."""
    # TODO: make it work for both U*IX and Windows
    SIOCGIFFLAGS = 0x8913
    null256 = "\0" * 256
    retval = set()

    if ifname in netifaces.interfaces():
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        result = fcntl.ioctl(s.fileno(), SIOCGIFFLAGS, ifname + null256)
        (flags,) = struct.unpack("H", result[16:18])
        for flag in rIFF:
            if flag & flags == flag:
                retval.add(rIFF[flag])
    return retval


def is_local_ipv4(addr):
    """This function returns True if 'addr' is a local IP V4 address."""
    if addr in local_ipv4():
        return True
    return False


def is_local_ipv6(addr):
    """This function returns True if 'addr' is a local IP V6 address."""
    if addr in local_ipv6():
        return True
    return False


def is_local_ip(addr):
    """This function returns True if 'addr' is a local IP V4 or V6 address."""
    return is_local_ipv4(addr) or is_local_ipv6(addr)


def local_ipv4():
    """This function returns a list of all local IP V4 addresses."""
    retval = []
    for interface in netifaces.interfaces():
        addresses = netifaces.ifaddresses(interface)
        for address in addresses[netifaces.AF_INET]:
            retval.append(address["addr"])
    return retval


def local_ipv6():
    """This function returns a list of all local IPV6 addresses."""
    retval = []
    for interface in netifaces.interfaces():
        addresses = netifaces.ifaddresses(interface)
        for address in addresses[netifaces.AF_INET6]:
            retval.append(address["addr"])
    return retval


def local_ip():
    """This function returns a list of all IP addresses."""
    return local_ipv4() + local_ipv6()


def get_domain_name():
    """This function returns the domain name."""
    retval = socket.getfqdn().replace(socket.gethostname(), "")
    if retval.startswith("."):
        retval = retval[1:]
    return retval


def str2bool(string):
    """This function converts a string to a boolean."""
    return string.lower() in ("yes", "y", "true", "t", "1")


class self_signed_certificate:
    """Self Signed Certificate, valid for 15 years.

    references:
        - https://stackoverflow.com/questions/27164354/create-a-self-signed-x509-certificate-in-python
        - https://stackoverflow.com/questions/24678308/how-to-find-location-with-ip-address-in-python
    """

    def __init__(self, email=None, sn=0):
        data = json.loads(
            urllib3.PoolManager()
            .request("GET", "http://ipinfo.io/json")
            .data.decode("utf-8")
        )
        self.key = OpenSSL.crypto.PKey()
        self.key.generate_key(OpenSSL.crypto.TYPE_RSA, 4096)
        self.certificate = OpenSSL.crypto.X509()
        self.certificate.get_subject().C = data["country"]
        self.certificate.get_subject().ST = data["region"]
        self.certificate.get_subject().L = f"{data['postal']} {data['city']}"
        self.certificate.get_subject().O = data["org"]
        self.certificate.get_subject().OU = f"{data['ip']}@{int(time.time())}"
        self.certificate.get_subject().CN = data["loc"]
        if email is None:
            self.certificate.get_subject().emailAddress = "fu@bar"
        else:
            self.certificate.get_subject().emailAddress = email
        self.certificate.set_serial_number(sn)
        self.certificate.gmtime_adj_notBefore(0)
        self.certificate.gmtime_adj_notAfter(15 * 365 * 24 * 60 * 60)
        self.certificate.set_issuer(self.certificate.get_subject())
        self.certificate.set_pubkey(self.key)
        self.certificate.sign(self.key, "sha512")

    def write(self, path=None):
        """This method will write the certificate and the private key to path."""
        if path is None:
            path = os.getcwd()
        path = os.path.abspath(path)

        if not os.path.exists(path):
            raise Exception(f"{path} does not exist")

        with open(os.path.join(path, "selfsigned.crt"), "wt") as f:
            f.write(
                OpenSSL.crypto.dump_certificate(
                    OpenSSL.crypto.FILETYPE_PEM, self.certificate
                ).decode("utf-8")
            )

        with open(os.path.join(path, "private.key"), "wt") as f:
            f.write(
                OpenSSL.crypto.dump_privatekey(
                    OpenSSL.crypto.FILETYPE_PEM, self.key
                ).decode("utf-8")
            )


def is_self_signed_certificate(cert_path):
    """This function returns True if 'cert' is a self signed certificate."""


if __name__ == "__main__":
    # ssc = self_signed_certificate()
    # ssc.write()
    ips = local_ip() + ["1.2.3.4"]
    for ip in ips:
        print(f"'{ip}' {('is not a local', 'is a local')[is_local_ip(ip)]} ip address")

    print(get_domain_name())



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
import json
import os
import socket
import time

import psutil
from spyder_remote_client.constants import SERVICE_TYPE
from spyder_remote_client.utils import str2bool
from zeroconf import ServiceBrowser
from zeroconf import Zeroconf

os.environ["SPYDER_REMOTE_DEBUG"] = "1"


class SpyderRemoteListener:
    """
    Spyder remote Zeroconf listener.

    Notes
    -----
    https://github.com/jstasiak/python-zeroconf/blob/master/zeroconf/__init__.py#L1448
    """

    def __init__(self):
        self.hosts = {}
        self.hosts_properties = {}

    def _verbose(self):
        return bool(os.environ.get("SPYDER_REMOTE_DEBUG"))

    def add_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service addition.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.
        """
        info = zeroconf_instance.get_service_info(service_type, name)
        new_info = {}
        for key, value in info.properties.items():
            new_info[key.decode()] = value.decode()

        self.hosts[name] = info
        self.hosts_properties[name] = new_info
        if self._verbose():
            print(f"adding service type '{service_type}' from '{name}'")
            print(info)

    def remove_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service removal.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.
        """
        if name in self.hosts:
            del self.hosts[name]
            del self.hosts_properties[name]

        if self._verbose():
            print(f"removing service type '{service_type}' from '{name}'")

    def update_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service update.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.
        """
        info = zeroconf_instance.get_service_info(service_type, name)
        self.hosts[name] = info

        if self._verbose():
            print(f"updating service type '{service_type}' from '{name}' to {info}")

    def get_hosts(self):
        """
        Return a dictionary with the host name as key, and the info associated
        with it as value.

        It is **not** simply a copy of self.hosts, we need to replace the
        local host with 'localhost' (and the interface).
        """
        my_addresses = {}
        my_interfaces = psutil.net_if_addrs()
        for interface in my_interfaces:
            for nic in my_interfaces[interface]:
                if nic.family in [socket.AF_INET, socket.AF_INET6]:
                    my_addresses[nic.address] = interface

        retval = {}
        for host in self.hosts:
            if len(self.hosts[host].addresses) != 1:
                raise Exception(f"Yup, not yet implemented ... {self.hosts[host]}")

            host_address = socket.inet_ntoa(self.hosts[host].addresses[0])
            properties = self.hosts_properties[host].copy()
            retval[host] = properties
            retval[host]["address"] = host_address
            retval[host]["port"] = self.hosts[host].port
            retval[host]["guest_account"] = properties["guest_account"]
            # retval[host]['guest_can_manage_environments'] = str2bool(properties['guest_can_manage_environments'])
            # retval[host]['status'] = properties['status']
            # retval[host]['local'] = host_address in my_addresses

        if self._verbose():
            print(retval)

        return retval


if __name__ == "__main__":
    zeroconf_instance = Zeroconf()
    listener = SpyderRemoteListener()
    browser = ServiceBrowser(zeroconf_instance, SERVICE_TYPE, listener)
    try:
        input("Press enter to exit...\n\n")
        time.sleep(2)
        print(listener.get_hosts())
    finally:
        zeroconf.close()



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
"""
Spyder Remote Container.
"""

from qtpy.QtCore import Signal
from spyder.api.translations import get_translation
from spyder.api.widgets.main_container import PluginMainContainer

from spyder_remote_client.spyder.widgets import RemoteConsoleDialog

# Localization
_ = get_translation("spyder_remote_client")


class SpyderRemoteContainer(PluginMainContainer):
    def __init__(self, name, plugin, parent=None):
        super().__init__(name, plugin, parent=parent)

        self._dialog = RemoteConsoleDialog(self)
        self._dialog.sig_connect_to_kernel.connect(self.sig_connect_to_kernel)
        self._dialog.setModal(True)

    # --- Signals
    # ------------------------------------------------------------------------
    sig_connect_to_kernel = Signal(object)
    """
    This signal is emitted with the json data to perform the connection to the
    remote spyder-kernel.

    Parameters
    ----------
    json_data: dict
        The kernel spec file with the information to perform the connection.
    """

    # --- PluginMainContainer API
    # ------------------------------------------------------------------------
    def setup(self):
        pass

    def on_option_update(self, options, value):
        pass

    def update_actions(self):
        pass

    # --- Public API
    # ------------------------------------------------------------------------
    def new_remote_console(self):
        """
        Connect to a remote console.
        """
        self._dialog.clear()
        self._dialog.setup()
        self._dialog.show()

    def close_all_kernels(self):
        """
        Close all kernels started on this session.
        """
        self._dialog.close_all_kernels()

    def close_services(self):
        """
        Close zeroconf services.
        """
        if self._dialog:
            self._dialog.close()

    @property
    def kernels(self):
        """Kernels started by Spyder remote client."""
        return self._dialog._kernels



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
"""
Spyder Remote Plugin.
"""

import json
import os
import tempfile
import uuid

from qtpy.QtGui import QIcon

# from spyder.api.plugins import Plugins, SpyderDockablePlugin
# from spyder.api.plugin_registration.decorators import on_plugin_available
# from spyder.config.base import get_translation

from spyder.api.plugins import Plugins, SpyderPluginV2
from spyder.api.translations import get_translation
from spyder.api.plugin_registration.decorators import on_plugin_available

from spyder_remote_client.spyder.container import SpyderRemoteContainer

# Localization
_ = get_translation("spyder_remote_client")


# --- Constants
# ----------------------------------------------------------------------------
class SpyderRemoteActions:
    NewRemoteConsole = "new_remote_console_action"
    CloseRemoteKernels = "close_remote_kernels"


# --- Plugin
# ----------------------------------------------------------------------------
class SpyderRemote(SpyderPluginV2):
    """
    Spyder Remote Plugin.
    """

    NAME = "spyder_remote"
    REQUIRES = [Plugins.IPythonConsole]
    CONTAINER_CLASS = SpyderRemoteContainer
    CONF_SECTION = NAME

    # --- SpyderPluginV2 API
    # ------------------------------------------------------------------------
    def get_name(self):
        return _("Spyder Remote Client")

    def get_description(self):
        return _("Discover Spyder kernels via zeroconf.")

    def get_icon(self):
        return QIcon()

    def on_initialize(self):
        container = self.get_container()
        self.new_remote_client_action = self.create_action(
            SpyderRemoteActions.NewRemoteConsole,
            text=_("New remote Console"),
            triggered=self.new_remote_console,
            register_shortcut=True,
        )
        self.close_all_kernels_action = self.create_action(
            SpyderRemoteActions.CloseRemoteKernels,
            text=_("Close remote kernels"),
            triggered=self.close_all_kernels,
            register_shortcut=True,
        )
        container.sig_connect_to_kernel.connect(self.start_remote_kernel)
        container.sig_connect_to_kernel.connect(lambda x: self.update_actions())
        self.close_all_kernels_action.setEnabled(False)

    @on_plugin_available(plugin=Plugins.IPythonConsole)
    def on_IPythonConsole_available(self):
        main_consoles_menu = self.main.consoles_menu_actions
        main_consoles_menu.insert(0, self.new_remote_client_action)
        main_consoles_menu.insert(1, self.close_all_kernels_action)

    def on_close(self, cancellable=True):
        self.close_all_kernels()
        self.close_services()
        return True

    def update_actions(self):
        self.close_all_kernels_action.setEnabled(bool(self.get_container().kernels))

    # --- API
    # ------------------------------------------------------------------------
    def close_all_kernels(self):
        """
        Close all kernels started on this session.
        """
        self.get_container().close_all_kernels()
        self.update_actions()
        self.sig_status_message_requested.emit(
            _("Spyder Remote Server: Closing all Kernels..."),
            5000,
        )

    def new_remote_console(self):
        """
        Connect to a remote console.
        """
        self.get_container().new_remote_console()

    def start_remote_kernel(self, json_data):
        """
        Parameters
        ----------
        json_data: dict
            The kernel spec file with the information to perform the
            connection.
        """
        connection_file = os.path.join(
            tempfile.tempdir,
            str(uuid.uuid4()) + ".json",
        )
        with open(connection_file, "w") as fh:
            fh.write(json.dumps(json_data["json_data"]))

        self.main.ipyconsole._create_client_for_kernel(
            connection_file, None, None, None
        )

    def close_services(self):
        """
        Close zeroconf services.
        """
        self.get_container().close_services()



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
"""
Spyder Remote zeroconf Qt listener.
"""

from qtpy.QtCore import QObject, Signal

from spyder_remote_client.discover import SpyderRemoteListener


class QSpyderRemoteListener(QObject, SpyderRemoteListener):
    """
    A Zeroconf server listener for spyder-kernels services.

    Connects the SpyderRemoteListener to Qt Signals.
    """

    sig_service_added = Signal(object, str, str)
    """
    This signal is emitted when a zeroconf service is added.

    Parameters
    ----------
    zeroconf_instance: Zeroconf
        The zerconf instance.
    service_type: str
        The service type. Example: '_sdk._tcp.local.'
    name: str
        The name of the host exposing the service.
    """

    sig_service_removed = Signal(object, str, str)
    """
    This signal is emitted when a zeroconf service is removed.

    Parameters
    ----------
    zeroconf_instance: Zeroconf
        The zerconf instance.
    service_type: str
        The service type. Example: '_sdk._tcp.local.'
    name: str
        The name of the host exposing the service.
    """

    sig_service_updated = Signal(object, str, str)
    """
    This signal is emitted when a zeroconf service is updated.

    Parameters
    ----------
    zeroconf_instance: Zeroconf
        The zerconf instance.
    service_type: str
        The service type. Example: '_sdk._tcp.local.'
    name: str
        The name of the host exposing the service.
    """

    def __init__(self):
        super().__init__()

    def add_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service addition.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.

        Notes
        -----
        This method emits a Qt signal.
        """
        super().add_service(zeroconf_instance, service_type, name)
        self.sig_service_added.emit(zeroconf_instance, service_type, name)

    def remove_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service addition.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.

        Notes
        -----
        This method emits a Qt signal.
        """
        super().remove_service(zeroconf_instance, service_type, name)
        self.sig_service_removed.emit(zeroconf_instance, service_type, name)

    def update_service(self, zeroconf_instance, service_type, name):
        """
        Handle a zeroconf service addition.

        Parameters
        ----------
        zeroconf_instance: Zeroconf
            The zerconf instance.
        service_type: str
            The service type. Example: '_sdk._tcp.local.'
        name: str
            The name of the host exposing the service.

        Notes
        -----
        This method emits a Qt signal.
        """
        super().update_service(zeroconf_instance, service_type, name)
        self.sig_service_updated.emit(zeroconf_instance, service_type, name)



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
import subprocess

from spyder_remote_client.spyder import DIALOG_UI_PATH

# Constants
def main():
    output = DIALOG_UI_PATH.replace(".ui", ".py")
    args = ["pyuic5", DIALOG_UI_PATH, "-o", output]
    p = subprocess.Popen(args)
    print(" ".join(args))
    stdout, stderr = p.communicate()
    print(stdout, stderr)


# TODO:
# Replace this line
# from PyQt5 import QtCore, QtGui, QtWidgets
# With
# from qtpy import QtCore, QtGui, QtWidgets


if __name__ == "__main__":
    main()



# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
"""
Spyder Remote Widgets.
"""

import json
import time

import zmq
from qtpy.QtCore import Signal
from qtpy.QtWidgets import QDialog, QMessageBox
from zeroconf import ServiceBrowser, Zeroconf

from spyder_remote_client.constants import SERVICE_TYPE
from spyder_remote_client.spyder.dialog import Ui_Dialog
from spyder_remote_client.spyder.discover import QSpyderRemoteListener
from spyder_remote_client.exceptions import SpyderRemoteServerException


class RemoteConsoleDialog(QDialog):

    # Signals
    # ------------------------------------------------------------------------
    sig_connect_to_kernel = Signal(object)
    """
    This signal is emitted with the json data to perform the connection to the
    remote spyder-kernel.

    Parameters
    ----------
    json_spec: dict
        The kernel spec file with the information to perform the connection.
    """

    def __init__(self, parent):
        super().__init__(parent)
        self._dialog_ui = Ui_Dialog()
        self._dialog_ui.setupUi(self)
        self._zeroconf_instance = Zeroconf()
        self._listener = QSpyderRemoteListener()
        self._kernels = []

        # Widgets
        self.host_combo = self._dialog_ui.spyderHosts
        self.user_combo = self._dialog_ui.user
        self.env_comvo = self._dialog_ui.condaEnvironments
        self.req = self._dialog_ui.requirements
        self.req_label = self._dialog_ui.label_6
        self.keyring_check = self._dialog_ui.keyring
        self.password = self._dialog_ui.password
        self.cancel_button = self._dialog_ui.cancelButton
        self.connect_button = self._dialog_ui.connectButton
        self.load_req_button = self._dialog_ui.findRequirements
        self.feedback_label = self._dialog_ui.feedback

        self.feedback_label.setText("")
        self.setWindowTitle("Connect to remote kernel")

        # Signals
        self.cancel_button.clicked.connect(self.close)
        self.connect_button.clicked.connect(self.connect)
        self.host_combo.currentIndexChanged.connect(self.change_host)

        self._listener.sig_service_added.connect(self._on_service_update)
        self._listener.sig_service_removed.connect(self._on_service_update)
        self._listener.sig_service_updated.connect(self._on_service_update)

        # Start zeroconf service browser
        self._browser = ServiceBrowser(
            self._zeroconf_instance,
            SERVICE_TYPE,
            self._listener,
        )

    def _on_service_update(self, zeroconf, service_type, service_name):
        """
        Update dialog on Zeroconf updates.

        Parameters
        ----------
        zeroconf : zeroconf.Zeroconf
            Zeroconf instance.
        service_type : str
            Zeroconf service type.
        service_name : str
            Zeroconf name type.
        """
        if self.isVisible():
            self.clear()
            self.setup()

    def _send(self, address, server_port, data):
        """
        Send data via zmq to Spyder Remote Server with address and port.

        Parameters
        ----------
        address : zeroconf.Zeroconf
            Spyder remote server address.
        server_port : str
            Spyder remote server port.
        data : dict
            Data to send to server.
        """
        received_message = None
        received_data = {}

        context = zmq.Context()
        socket = context.socket(zmq.REQ)
        full_address = f"tcp://{address}:{server_port}"
        socket.setsockopt(zmq.LINGER, 0)
        socket.connect(full_address)

        if socket.poll(timeout=1000, flags=zmq.POLLOUT) != 0:
            send_message = json.dumps(data)
            print("Sending request %s …" % send_message)
            socket.send_string(send_message, flags=zmq.NOBLOCK)
            if socket.poll(timeout=5000, flags=zmq.POLLIN) != 0:
                try:
                    received_message = socket.recv(flags=zmq.NOBLOCK)
                except zmq.Again:
                    pass

        if received_message:
            print("Received message %s …" % received_message)
            received_data = json.loads(received_message.decode("utf-8"))

        socket.disconnect(full_address)
        socket.close(linger=0)
        context.term()

        return received_data

    # --- Qt overrides
    # ------------------------------------------------------------------------
    def close(self):
        """
        Override Qt method.

        Close the zeroconf service browser.
        """
        self._zeroconf_instance.close()
        return super().close()

    # --- Public API
    # ------------------------------------------------------------------------
    def close_all_kernels(self):
        """
        Close all kernels started on this session.
        """
        hosts = self._listener.get_hosts()
        if self._kernels:
            for _host, properties in hosts.items():
                address = properties["address"]
                server_port = properties["server_port"]
                data = {"kernel": {"command": "close_all"}}
                self._send(address, server_port, data)

            self._kernels = []

    def connect(self):
        """
        Connect to selected kernel.
        """
        self.feedback_label.setText("Requesting new remote kernel...")

        properties = self.host_combo.currentData()
        address = properties["address"]
        server_port = properties["server_port"]

        prefix = self.env_comvo.currentData()
        data = {"kernel": {"command": "start", "prefix": prefix}}
        received_data = self._send(address, server_port, data)

        if received_data:
            self.feedback_label.setText("Starting remote kernel...")
            error = data.get("error")
            if error:
                self.feedback_label.setText("Spyder remote server error")
                raise SpyderRemoteServerException(error)
            else:
                self._kernels.append(prefix)
                self.sig_connect_to_kernel.emit(received_data)

            self.accept()
        else:
            self.clear()
            self.feedback_label.setText(f"Spyder remote server not responding!")
            self.connect_button.setEnabled(False)

    def clear(self):
        """Clear comboboxes and information text."""
        self.host_combo.clear()
        self.user_combo.clear()
        self.env_comvo.clear()
        self.feedback_label.setText("")

    def setup(self):
        """
        Setup the comboboxes.
        """
        self.req.setVisible(False)
        self.req_label.setVisible(False)
        self.load_req_button.setVisible(False)
        self.keyring_check.setDisabled(True)
        self.password.setDisabled(True)

        hosts = self._listener.get_hosts()

        for _host, properties in hosts.items():
            self.host_combo.addItem(properties["name"], properties)

        self.connect_button.setEnabled(bool(hosts))

    def change_host(self):
        """
        Update user and environment when a different host has been selected.
        """
        self.env_comvo.clear()
        self.user_combo.clear()
        _current_text = self.host_combo.currentText()
        properties = self.host_combo.currentData()
        if properties is not None:
            self.user_combo.addItem(properties["guest_account"])
            for key, value in properties.items():
                if key.startswith("conda_env_") and key.endswith("_yes"):
                    env_name = value.split("/")[-1]
                    self.env_comvo.addItem(env_name, value)
